#!/bin/bash
#
# File: create-release-makefiles.sh
#
# Creates the release mode build cmake scripts.
#
# Date: Thu Apr 20, 10:32:00 EST 2017.
# Author: manuel.arguelles
#
# Revision: 1
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#

ARCH=""
CMAKELISTS_DIR="."
BUILD_DIR="build"
CMAKE_USER_OPTIONS=""

for i in "$@"
do
  case $i in
    arch=*)
      ARCH="${i#*=}"
      shift
      ;;
    cmakelists=*)
      CMAKELISTS_DIR="${i#*=}"
      shift
      ;;
    builddir=*)
      BUILD_DIR="${i#*=}"
      shift
      ;;
    *)
      CMAKE_USER_OPTIONS="${CMAKE_USER_OPTIONS} $i"
      shift
      ;;
  esac
done

CMAKE_USER_OPTIONS="${CMAKE_USER_OPTIONS} -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr"

if [ -n "${ARCH}" ]
then
  CMAKE_OPTIONS=(-DBUILD_ARCHITECTURE=${ARCH} -DCMAKE_TOOLCHAIN_FILE=cmake/CrossToolchain.cmake "${CMAKE_OPTIONS}")
fi

hash ninja 2>/dev/null && CMAKE_USER_OPTIONS="-GNinja ${CMAKE_USER_OPTIONS}"

rm -rf ${BUILD_DIR}/CMakeCache.txt ${BUILD_DIR}/CMakeFiles
cmake "${CMAKE_OPTIONS[@]}" ${CMAKE_USER_OPTIONS} -B${BUILD_DIR} -H${CMAKELISTS_DIR}
