/**
 * @file Signed.cxx
 * @brief Body class for Signed file
 */

#include "Signed.hxx"
#include "XmlSigned.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * Signed::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Signed", GETTEXT_PACKAGE );

Signed::Signed()
    : SecurityBase()
{

}

Signed::~Signed()
{

}

void Signed::signBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut, Signed::SignedType signedType ) throw ( SignedException )
{
    bufferOut.Clear();

    if ( signedType == Signed::NOT_SIGNED )
    {
        bufferOut.AddData ( bufferIn );
        return;
    }

    if ( certificate == NULL )
    {
        logger->Debug ( "Error - Sign Dom - certificate NULL" );
        throw SignedException ( _ ( "Smartmatic.System.Security.Signed.ErrorSignedCert" ),
                                N_ ( "Smartmatic.System.Security.Signed.ErrorSignedCert" ), logger );
    }

    XmlSigned xmlSigned ( bufferIn );
    bool validate = false;

    if ( signedType == Signed::FULL_SIGNED )
    {
        validate = xmlSigned.signBuffer ( certificate->LoadCertifiedPrivateKey(), bufferOut, certificate->LoadCertifiedX509() );
    }
    else
    {
        validate = xmlSigned.signBuffer ( certificate->LoadCertifiedPrivateKey(), bufferOut, NULL );
    }

    if ( ! validate )
    {
        throw SignedException (
            _ ( "Smartmatic.System.Security.Signed.ErrorSignedCert" ),
            N_ ( "Smartmatic.System.Security.Signed.ErrorSignedCert" ),
            logger );
    }
}

bool Signed::verifyBuffer ( const SafeBuffer & buffer ) throw ( SignedException )
{
    XmlSigned xmlSigned ( buffer );
    bool validate = xmlSigned.verifyBuffer ( certificate ? certificate->LoadCertifiedX509() : NULL );

    if ( ! validate )
    {
        throw SignedException (
            _ ( "Smartmatic.System.Security.Signed.ErrorValidatingDigitalXmlSignature" ),
            N_ ( "Smartmatic.System.Security.Signed.ErrorValidatingDigitalXmlSignature" ),
            logger );
    }

    return validate;
}
