/**
 * 
 */
package com.smartmatic.security;

import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.CertStoreException;

import org.apache.commons.io.FileUtils;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openssl.PEMDecryptorProvider;
import org.bouncycastle.openssl.PEMEncryptedKeyPair;
import org.bouncycastle.openssl.PEMKeyPair;
import org.bouncycastle.openssl.PEMParser;
import org.bouncycastle.openssl.jcajce.JcaPEMKeyConverter;
import org.bouncycastle.openssl.jcajce.JcePEMDecryptorProviderBuilder;
import org.dom4j.DocumentException;

/**
 * @author marcel.mancini
 * 
 */
public class X7SEnvelopeUtility {

	/**
	 * 
	 */
	private static String x7sFilesPath;

	/**
	 * 
	 */
	private static Boolean areEncrypted;

	/**
	 * 
	 */
	private static String privateKeyPath;

	/**
	 * 
	 */
	private static String privateKey;

	/**
	 * 
	 */
	private static boolean checked;

	/**
	 * 
	 * @param token
	 * @return
	 */
	private static String[] getPairKeyValue(String keyName, String token) {
		String[] tokenSplited = token.split("=");
		if (!keyName.equals(tokenSplited[0].replaceAll("-", ""))) {
			throw new IllegalStateException("Wrong parameter key or value [ '"
					+ keyName + "' ]");
		}

		if (tokenSplited.length == 2) {
			return tokenSplited;
		}
		return null;
	}

	/**
	 * 
	 * @param file
	 * @return
	 * @throws IOException
	 */
	static PrivateKey getPrivateKey(File file) throws IOException {
		File privKeyFile = new File(file.getAbsolutePath());

		if (!privKeyFile.exists()) {
			throw new IOException("Invalid path " + file.getAbsolutePath());
		}

		Security.addProvider(new BouncyCastleProvider());
		PEMParser pemParser = new PEMParser(new FileReader(privKeyFile));
	    Object object = pemParser.readObject();
	    PEMDecryptorProvider decProv = new JcePEMDecryptorProviderBuilder().build(privateKey.toCharArray());
	    JcaPEMKeyConverter converter = new JcaPEMKeyConverter().setProvider("BC");
	    KeyPair kp;
	     if (object instanceof PEMEncryptedKeyPair) {
	        System.out.println("Encrypted key - we will use provided password");
	        kp = converter.getKeyPair(((PEMEncryptedKeyPair) object).decryptKeyPair(decProv));
	    } else {
	        System.out.println("Unencrypted key - no password needed");
	        kp = converter.getKeyPair((PEMKeyPair) object);
	    }
		PrivateKey privKey = kp.getPrivate();
		pemParser.close();
		return privKey;
	}

	/**
	 * 
	 * @param args
	 */
	static void checkParameters(String[] args) {
		String encrypted = null;
		String[] keyPair = args.length > 0 ? getPairKeyValue("x7spath", args[0])
				: null;

		if (keyPair != null) {
			x7sFilesPath = keyPair[1];
		}

		keyPair = args.length > 1 ? getPairKeyValue("encrypted", args[1])
				: null;
		if (keyPair != null) {
			encrypted = keyPair[1];
			areEncrypted = Boolean.valueOf(encrypted);
		}

		keyPair = args.length > 2 ? getPairKeyValue("privatekeypath", args[2])
				: null;
		if (keyPair != null) {
			privateKeyPath = keyPair[1];
		}

		keyPair = args.length > 3 ? getPairKeyValue("pwd", args[3]) : null;
		if (keyPair != null) {
			privateKey = keyPair[1];
		}

		if ((x7sFilesPath == null) || (encrypted == null)) {
			throw new IllegalStateException("Invalid parameters ["
					+ ((x7sFilesPath == null) ? " 'x7sFilesPath' " : "")
					+ ((encrypted == null) ? " 'encrypted' " : "") + "]");
		}

		if ((areEncrypted) && (privateKeyPath == null || privateKey == null)) {
			throw new IllegalStateException("Invalid required parameters ["
					+ ((privateKeyPath == null) ? " 'privatekeypath' " : "")
					+ ((privateKey == null) ? " 'pwd' " : "") + "]");
		}
		
		checked = true;
	}

	/**
	 * 
	 * @param args
	 * @throws IOException
	 * @throws NoSuchAlgorithmException
	 * @throws NoSuchProviderException
	 * @throws CertStoreException
	 * @throws CMSException
	 * @throws DocumentException
	 * @throws InterruptedException
	 */
	public static void main(String[] args) throws IOException,
			NoSuchAlgorithmException, NoSuchProviderException,
			CertStoreException, CMSException, DocumentException,
			InterruptedException {

		try {
			PrivateKey receiverPrivateKey = null;
			checked = false;
			
			checkParameters(args);

			if (areEncrypted) {
				receiverPrivateKey = getPrivateKey(new File(privateKeyPath));
			}

			System.out.println("Using root path [ " + x7sFilesPath + " ]");

			Thread.sleep(1000);

			File rootFolder = new File(x7sFilesPath);
			File[] x7sFiles = rootFolder.listFiles(new FilenameFilter() {

				@Override
				public boolean accept(File arg0, String arg1) {
					return arg1.endsWith("X7S");
				}
			});

			System.out.println("It were found ( " + x7sFiles.length
					+ " ) X7S files");
			Thread.sleep(1000);

			for (File file : x7sFiles) {
				System.out.println("Reading file = " + file.getAbsolutePath());
				Thread.sleep(1000);

				byte[] readSignedData = null;

				if (areEncrypted) {

					byte[] readEncryptedData = FileUtils
							.readFileToByteArray(new File(rootFolder, file
									.getName()));
					readSignedData = X7SEnvelopeHelper.decrypt(
							readEncryptedData, receiverPrivateKey);
				} else {

					readSignedData = FileUtils.readFileToByteArray(new File(
							rootFolder, file.getName()));
				}
				boolean valid = X7SEnvelopeHelper.verify(readSignedData);

				if (valid) {
					System.out.println("--Valid file--");
					Thread.sleep(500);
					byte[] originalContent = X7SEnvelopeHelper
							.unsealedEnvelope(readSignedData);
					System.out.println("--Reading its content--");
					Thread.sleep(500);
					String plainContent = new String(originalContent);
					System.out.println("--Writing its PLAIN content--");
					Thread.sleep(500);
					FileUtils.writeByteArrayToFile(
							new File(rootFolder, file.getName() + ".unsealed"),
							originalContent);
					byte[] pdf = X7SEnvelopeHelper.getRawData("SignedPV",
							plainContent);
					System.out.println("--Writing its PDF content--");
					Thread.sleep(500);
					FileUtils.writeByteArrayToFile(
							new File(rootFolder, file.getName()
									+ ".unsealed.pdf"), pdf);
					byte[] raw = X7SEnvelopeHelper.getRawData("FormatEML",
							plainContent);
					System.out.println("--Writing its EML content--");
					Thread.sleep(500);
					FileUtils.writeByteArrayToFile(
							new File(rootFolder, file.getName() + ".unsealed"),
							raw);
					System.out.println("Done file = " + file.getAbsolutePath());
					Thread.sleep(1000);
					System.out.println(" ");
				} else {
					System.err.println("--Invalid file--");
				}
			}
		} catch (Throwable e) {
			System.err.println("There was an error: [caused by = "
					+ e.getMessage() + "]");
			if (!checked) {
				System.out
						.println("Usage: java -jar X7SEnvelopeUtility.jar [-x7spath=PATH_TO_LOAD_AND_UNSEALED_FILES] | {[-encrypted= {true | false}] [-privatekeypath=PATH_TO_LOAD_PRIVATE_KEY] [-pwd=PRIVATE_KEY_PASSWORD]}");
			}
			System.exit(1);
		}
	}
}
