#!/bin/sh
#
# USAGE: Generate Jail for OS.
#
# EXAMPLE: ./smtt-generate-jail.sh <CHROOT_PATH>
#
# File: smtt-generate-jail.sh
#
# Date: Tue 22 jun 2017 18:45:23
# Author: gustavo.garcia
#
# Revision 1
#
# Copyright 2017 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.

CHROOTFOLDER=$1
OS_BUILDER=$(dirname $(readlink -f $0))/../
SMTT_PACKAGES="compiled-pkgs"
VERSION=$(cat $OS_BUILDER/VERSION)

if [ -z "$CHROOTFOLDER" ]; then
   echo "Please define the CHROOT folder"
   exit
fi

mkdir -p $CHROOTFOLDER
CHROOT_FULLPATH="$(readlink -f $CHROOTFOLDER)"
debootstrap --arch=i386 --variant=minbase trusty $CHROOTFOLDER http://pa.archive.ubuntu.com/ubuntu/

# Set Box Version Number
mkdir $CHROOTFOLDER/etc/smtt
echo 'OS Base: Ubuntu Trusty 14.04' > $CHROOTFOLDER/etc/smtt/smtt-os-version
echo "Version: $VERSION" >> $CHROOTFOLDER/etc/smtt/smtt-os-version

echo 'deb http://pa.archive.ubuntu.com/ubuntu trusty main restricted universe'          > $CHROOTFOLDER/etc/apt/sources.list

echo "I: Mounting dev, sys and proc filesystems"
mount --bind /dev/ $CHROOTFOLDER/dev
mount -t proc proc $CHROOTFOLDER/proc
mount -t sysfs sysfs $CHROOTFOLDER/sys

chroot $CHROOTFOLDER apt update

echo "I: Installing OS Dependencies"
chroot $CHROOTFOLDER apt-get install -y --no-install-recommends \
libpcsclite-dev \
libssl-dev \
symlinks \
libcups2-dev \
libusb-1.0-0-dev \
libqrencode-dev \
libb64-dev \
libcupsimage2-dev \
libx11-dev \
libdrm-dev \
x11proto-xf86dri-dev \
xserver-xorg-dev \
libpixman-1-dev

chroot $CHROOTFOLDER symlinks -rc /

TOOLCHAIN_SYSROOT=$CHROOT_FULLPATH ./scripts/smtt-compile-packages.sh -e $CHROOT_FULLPATH
if [ "$?" = "1" ]; then
   echo "Error: smtt-compile-packages.sh"
   umount -l $CHROOTFOLDER/dev
   umount $CHROOTFOLDER/proc
   umount $CHROOTFOLDER/sys
   exit 1
fi

cp $SMTT_PACKAGES/*.deb $CHROOTFOLDER/root/

cat <<EOT >> $CHROOTFOLDER/root/run.sh
#!/bin/sh -e
dpkg -i --force-overwrite /root/*.deb
EOT
if [ "$?" = "1" ]; then
   echo "Error: dpkg failed during installation"
   umount -l $CHROOTFOLDER/dev
   umount $CHROOTFOLDER/proc
   umount $CHROOTFOLDER/sys
   exit 1
fi

chroot $CHROOTFOLDER sh /root/run.sh

chroot $CHROOTFOLDER apt-get clean

sync
echo "Generation Completed: umounting filesystem folders"
umount -l $CHROOTFOLDER/dev
umount $CHROOTFOLDER/proc
umount $CHROOTFOLDER/sys

