/**
 * CryptographicProviderStaticException.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 04:47:27 $)
 *
 * Copyright (c) 2004 Smartmatic Intl.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Intl. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Intl.
 */

package com.smartmatic.waffle.security.lang;

import com.smartmatic.waffle.lang.BaseRuntimeException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * <p>
 * Class used to indicate that a exception occurred because exist a
 * configuration error in the system. When this exception is thrown exists a
 * problem in the configuration of the server.
 * </p>
 * <p>
 * Some scenarios when this exception is thrown are:
 * <ul>
 * <li>Does not exist a cryptographic provider that support the algorithms used
 * by the application</li>
 * <li>Does not exist the file where a key is located</li>
 * </ul>
 * </p>
 * 
 * @version 1.0
 * @author Ithamar Torres
 */

public class CryptographicProviderStaticException extends BaseRuntimeException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * <p>
	 * Constructor for an instance of this class having only the enumerated
	 * cause of the exception.
	 * </p>
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */

	public CryptographicProviderStaticException(IExceptionInfo key) {
		super(key);
	}

	public CryptographicProviderStaticException(String msg, Throwable e) {
		super(msg, e);
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having both the enumerated
	 * cause of the exception and the nested <code>Throw able</code> that might
	 * have triggered this exception.
	 * </p>
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param throwable
	 *            a nested exception
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */

	public CryptographicProviderStaticException(IExceptionInfo key,
			Throwable cause) {
		super(key, cause);
	}

	/**
	 * Creates a new instance of a CryptographicProviderStaticException with the
	 * received parameters
	 * 
	 * @param cause
	 *            The cause of the exception
	 */
	public CryptographicProviderStaticException(Throwable cause) {
		super(cause);
	}

}
