/**
 * SignValidatorTest.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.security.sign;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import org.apache.commons.io.FileUtils;
import org.apache.xml.security.signature.XMLSignature;
import org.junit.Before;
import org.junit.Test;

import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;

/**
 * Class used to test methods from SignValidator class.
 */
public class SignValidatorTest {

	/** The text signed. */
	String textSigned;

	/** The x509cert to validate. */
	X509Certificate x509cert;

	/**
	 * Test SetUp.
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {

		byte[] PKCS12Bytes = FileUtils
				.readFileToByteArray(new File(
						"test/com/smartmatic/waffle/security/test/testResources/test_cert.p12"));

		String textToSign = FileUtils
				.readFileToString(new File(
						"test/com/smartmatic/waffle/security/test/testResources/xml_to_test_signing.xml"));

		String caPassword = "bel2013";

		String caAlias = "ecm";

		String signatureAlgorithm = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;

		PrivateKey certificateCA = AsymmetricKeyManager.getPrivateKey(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias,
				caPassword);

		this.x509cert = AsymmetricKeyManager.getCertificate(
				new ByteArrayInputStream(PKCS12Bytes), caPassword, caAlias);

		SignGenerator signGenerator = new SignGenerator(this.x509cert,
				certificateCA, signatureAlgorithm);

		StringBuilder stringBuilder = new StringBuilder(textToSign);

		signGenerator.signText(stringBuilder);

		this.textSigned = stringBuilder.toString();
	}

	/**
	 * Test validate sign and get x509 certificate.
	 */
	@Test
	public void testValidateSignAndGetX509Certificate() throws Exception {

		// Arrange
		X509Certificate x509cert = SignValidator
				.validateSignAndGetX509Certificate(this.textSigned);

		// Act
		if (x509cert == null) {
			throw new RuntimeException("Data was not validated");
		}

		// Assert
		assertEquals(x509cert, this.x509cert);

	}

	/**
	 * Test get certificates validated.
	 */
	@Test
	public void testGetCertificatesValidated() throws Exception {

		// Arrange
		int certificatesValidated = SignValidator
				.getCertificatesValidated(this.textSigned);

		// Act
		if (certificatesValidated == 0) {
			throw new RuntimeException("Data was not validated");
		}

		// Assert
		assertEquals(certificatesValidated, 1);

	}

	/**
	 * Test validate signature.
	 */
	@Test
	public void testValidateSignature() throws Exception {

		// Arrange
		boolean certificatesValidated = SignValidator
				.validateSignature(this.textSigned);

		// Act
		if (!certificatesValidated) {
			throw new RuntimeException("Data was not validated");
		}

		// Assert
		assertTrue(certificatesValidated);
	}

}
